@@ -25,7 +25,7 @@ struct ast_sip_cli_context;
  * \internal
  * \brief Initialize the configuration for res_pjsip
  */
-int ast_res_pjsip_initialize_configuration(const struct ast_module_info *ast_module_info);
+int ast_res_pjsip_initialize_configuration(void);
 
 /*!
  * \internal
@@ -232,24 +232,6 @@ int ast_res_pjsip_init_message_filter(void);
  */
 int ast_res_pjsip_init_contact_transports(void);
 
-/*!
- * \internal
- * \brief Initialize outbound authentication support
- *
- * \retval 0 Success
- * \retval non-zero Failure
- */
-int internal_sip_initialize_outbound_authentication(void);
-
-/*!
- * \internal
- * \brief Destroy outbound authentication support
- *
- * \retval 0 Success
- * \retval non-zero Failure
- */
-void internal_sip_destroy_outbound_authentication(void);
-
 /*!
  * \internal
  * \brief Initialize system configuration
@@ -271,6 +253,12 @@ void ast_sip_destroy_system(void);
  */
 void ast_sip_initialize_dns(void);
 
+/*!
+ * \internal
+ * \brief Initialize our own resolver support
+ */
+void ast_sip_initialize_resolver(void);
+
 /*!
  * \internal
  * \brief Initialize global configuration
@@ -335,56 +323,8 @@ int sip_cli_print_global(struct ast_sip_cli_context *context);
  */
 int sip_cli_print_system(struct ast_sip_cli_context *context);
 
-/*!
- * \internal
- * \brief Used by res_pjsip.so to register a service without adding a self reference
- */
-int internal_sip_register_service(pjsip_module *module);
-
-/*!
- * \internal
- * \brief Used by res_pjsip.so to unregister a service without removing a self reference
- */
-int internal_sip_unregister_service(pjsip_module *module);
-
-/*!
- * \internal
- * \brief Used by res_pjsip.so to register a supplement without adding a self reference
- */
-void internal_sip_register_supplement(struct ast_sip_supplement *supplement);
-
-/*!
- * \internal
- * \brief Used by res_pjsip.so to unregister a supplement without removing a self reference
- */
-int internal_sip_unregister_supplement(struct ast_sip_supplement *supplement);
-
-/*!
- * \internal
- * \brief Used by res_pjsip.so to register an endpoint formatter without adding a self reference
- */
-void internal_sip_register_endpoint_formatter(struct ast_sip_endpoint_formatter *obj);
-
-/*!
- * \internal
- * \brief Used by res_pjsip.so to unregister a endpoint formatter without removing a self reference
- */
-int internal_sip_unregister_endpoint_formatter(struct ast_sip_endpoint_formatter *obj);
-
 struct ast_sip_session_supplement;
 
-/*!
- * \internal
- * \brief Used by res_pjsip.so to register a session supplement without adding a self reference
- */
-void internal_sip_session_register_supplement(struct ast_sip_session_supplement *supplement);
-
-/*!
- * \internal
- * \brief Used by res_pjsip.so to unregister a session supplement without removing a self reference
- */
-int internal_sip_session_unregister_supplement(struct ast_sip_session_supplement *supplement);
-
 /*!
  * \internal
  * \brief Finds or creates contact_status for a contact
@@ -416,18 +356,6 @@ int ast_sip_initialize_scheduler(void);
  */
 int ast_sip_destroy_scheduler(void);
 
-/*!
- * \internal
- * \brief Add a reference to the res_pjsip module
- */
-void internal_res_pjsip_ref(void);
-
-/*!
- * \internal
- * \brief Remove a reference from the res_pjsip module
- */
-void internal_res_pjsip_unref(void);
-
 /*!
  * \internal
  * \brief Determines if a uri will still be valid after an asterisk restart
